@@ -478,7 +478,7 @@ sub get_version {
     my $r = $self->runquery("SELECT version()");
     my $v = $r->[0]->[0];
     die "Unable to detect PostgreSQL version\n"
-        unless ($v =~ /^PostgreSQL (\d+)\.(\d+)\.(\d+) on/);
+        unless ($v =~ /^PostgreSQL (\d+)\.(\d+)(\.\d+|devel) on/);
     $self->{detected_version} = "$1.$2";
 }
 
